@@ -12,11 +12,12 @@
     :license: BSD, see LICENSE for more details.
 """
 
-from eve.methods import get, getitem, post, patch, delete, delete_resource
+from eve.methods import get, getitem, post, patch, delete, delete_resource, put
 from eve.methods.common import ratelimit
 from eve.render import send_response
 from eve.auth import requires_auth
-from eve.utils import resource_uri, config, request_method
+from eve.utils import resource_uri, config, request_method, \
+    debug_error_message
 from flask import abort
 
 
@@ -57,6 +58,9 @@ def item_endpoint(url, **lookup):
     :param url: the url that led here
     :param lookup: the query
 
+    .. versionchanged:: 0.1.0
+       Support for PUT method.
+
     .. versionchanged:: 0.0.7
        Using 'utils.request_method' helper function now.
 
@@ -70,6 +74,8 @@ def item_endpoint(url, **lookup):
         response = getitem(resource, **lookup)
     elif method == 'PATCH':
         response = patch(resource, **lookup)
+    elif method == 'PUT':
+        response = put(resource, **lookup)
     elif method == 'DELETE':
         response = delete(resource, **lookup)
     elif method == 'OPTIONS':
@@ -83,11 +89,18 @@ def item_endpoint(url, **lookup):
 @requires_auth('home')
 def home_endpoint():
     """ Home/API entry point. Will provide links to each available resource
+
+    .. versionchanged:: 0.1.0
+       Support for optional HATEOAS.
     """
-    response = {}
-    links = []
-    for resource in config.DOMAIN.keys():
-        links.append({'href': '%s' % resource_uri(resource),
-                      'title': '%s' % config.URLS[resource]})
-    response['_links'] = {'child': links}
-    return send_response(None, (response,))
+    if config.HATEOAS:
+        response = {}
+        links = []
+        for resource in config.DOMAIN.keys():
+            links.append({'href': '%s' % resource_uri(resource),
+                          'title': '%s' % config.URLS[resource]})
+        response['_links'] = {'child': links}
+        return send_response(None, (response,))
+    else:
+        abort(404, debug_error_message("HATEOAS is disabled so we have no data"
+                                       " to display at the API homepage."))
