@@ -6,10 +6,10 @@
 
     This package implements the MongoDB data layer.
 
-    :copyright: (c) 2012 by Nicola Iarocci.
+    :copyright: (c) 2013 by Nicola Iarocci.
     :license: BSD, see LICENSE for more details.
 """
 
 # flake8: noqa
-from mongo import Mongo
-from validation import Validator
+from eve.io.mongo.mongo import Mongo
+from eve.io.mongo.validation import Validator
