@@ -11,8 +11,11 @@
 """
 
 import ast
+import itertools
+from bson.errors import InvalidId
 import simplejson as json
 import pymongo
+import sys
 from flask import abort
 from flask.ext.pymongo import PyMongo
 from datetime import datetime
@@ -136,23 +139,70 @@ class Mongo(DataLayer):
         :param resource: resource name.
         :param **lookup: lookup query.
 
+        .. versionchanged:: 0.1.0
+           ID_FIELD to ObjectID conversion is done before `_datasource_ex` is
+           called.
+
         .. versionchanged:: 0.0.6
            Only retrieve fields in the resource schema
 
         .. versionchanged:: 0.0.4
            retrieves the target collection via the new config.SOURCES helper.
         """
-        datasource, filter_, projection = self._datasource_ex(resource, lookup)
+        if config.ID_FIELD in lookup:
+            try:
+                lookup[ID_FIELD] = ObjectId(lookup[ID_FIELD])
+            except (InvalidId, TypeError):
+                # Returns a type error when {'_id': {...}}
+                pass
 
-        try:
-            if config.ID_FIELD in filter_:
-                filter_[ID_FIELD] = ObjectId(filter_[ID_FIELD])
-        except:
-            pass
+        datasource, filter_, projection = self._datasource_ex(resource, lookup)
 
         document = self.driver.db[datasource].find_one(filter_, projection)
         return document
 
+    def find_list_of_ids(self, resource, ids, client_projection=None):
+        """Retrieves a list of documents from the collection given
+        by `resource`, matching the given list of ids.
+
+        This query is generated to *preserve the order* of the elements
+        in the `ids` list. An alternative would be to use the `$in` operator
+        and accept non-dependable ordering for a slight performance boost
+        see <https://jira.mongodb.org/browse/SERVER-7528?focusedCommentId=
+        181518&page=com.atlassian.jira.plugin.system.issuetabpanels:comment
+        -tabpanel#comment-181518>
+
+        To preserve order, we use a query of the form
+            db.collection.find( { $or:[ { _id:ObjectId(...) },
+                { _id:ObjectId(...) }...] } )
+
+        Instead of the simpler
+            {'_id': {'$in': ids}}
+
+        -- via http://stackoverflow.com/a/13185509/1161906
+
+        :param resource: resource name.
+        :param ids: a list of ObjectIds corresponding to the documents
+        to retrieve
+        :param client_projection: a specific projection to use
+        :return: a list of documents matching the ids in `ids` from the
+        collection specified in `resource`
+
+        .. versionadded:: 0.1.0
+        """
+        query = {'$or': [
+            {'_id': id_} for id_ in ids
+        ]}
+
+        datasource, spec, projection = self._datasource_ex(
+            resource, query=query, client_projection=client_projection
+        )
+
+        documents = self.driver.db[datasource].find(
+            spec=spec, fields=projection
+        )
+        return documents
+
     def insert(self, resource, doc_or_docs):
         """Inserts a document into a resource collection.
 
@@ -212,6 +262,26 @@ class Mongo(DataLayer):
                 'pymongo.errors.OperationFailure: %s' % e
             ))
 
+    def replace(self, resource, id_, document):
+        """Replaces an existing document.
+
+        .. versionadded:: 0.1.0
+        """
+        datasource, filter_, _ = self._datasource_ex(resource,
+                                                     {ID_FIELD: ObjectId(id_)})
+
+        # TODO consider using find_and_modify() instead. The document might
+        # have changed since the ETag was computed. This would require getting
+        # the original document as an argument though.
+        try:
+            self.driver.db[datasource].update(filter_, document,
+                                              **self._wc(resource))
+        except pymongo.errors.OperationFailure as e:
+            # see comment in :func:`insert()`.
+            abort(500, description=debug_error_message(
+                'pymongo.errors.OperationFailure: %s' % e
+            ))
+
     def remove(self, resource, id_=None):
         """Removes a document or the entire set of documents from a collection.
 
@@ -240,20 +310,109 @@ class Mongo(DataLayer):
                 'pymongo.errors.OperationFailure: %s' % e
             ))
 
+    # TODO: The next three methods could be pulled out to form the basis
+    # of a separate MonqoQuery class
+
+    def combine_queries(self, query_a, query_b):
+        """
+        Takes two db queries and applies db-specific syntax to produce
+        the intersection
+
+        This is used because we can't just dump one set of query operators
+        into another.
+
+        Consider for example if the dataset contains a custom datasource
+        pattern like --
+           'filter': {'username': {'$exists': True}}
+
+        If we simultaneously try to filter on the field `username`,
+        then doing
+            query_a.update(query_b)
+        would lose information.
+
+        This implementation of the function just combines everything in the
+        two dicts using the `$and` operator.
+
+        Note that this is exactly same as performing dict.update() except
+        when multiple operators are operating on the /same field/.
+
+        Example:
+            combine_queries({'username': {'$exists': True}},
+                            {'username': 'mike'})
+        {'$and': [{'username': {'$exists': True}}, {'username': 'mike'}]}
+
+        .. versionadded: 0.1.0
+           Support for intelligent combination of db queries
+        """
+        # Chain the operations with the $and operator
+        return {
+            '$and': [
+                {k: v} for k, v in itertools.chain(query_a.items(),
+                                                   query_b.items())
+            ]
+        }
+
+    def get_value_from_query(self, query, field_name):
+        """ For the specified field name, parses the query and returns
+        the value being assigned in the query.
+
+        For example,
+            get_value_from_query({'_id': 123}, '_id')
+        123
+
+        This mainly exists to deal with more complicated compound queries
+            get_value_from_query(
+                {'$and': [{'_id': 123}, {'firstname': 'mike'}],
+                '_id'
+            )
+        123
+
+        .. versionadded: 0.1.0
+           Support for parsing values embedded in compound db queries
+        """
+        if field_name in query:
+            return query[field_name]
+        elif '$and' in query:
+            for condition in query['$and']:
+                if field_name in condition:
+                    return condition[field_name]
+        raise KeyError
+
+    def query_contains_field(self, query, field_name):
+        """ For the specified field name, does the query contain it?
+        Used know whether we need to parse a compound query
+
+        .. versionadded: 0.1.0
+           Support for parsing values embedded in compound db queries
+        """
+        try:
+            self.get_value_from_query(query, field_name)
+        except KeyError:
+            return False
+        return True
+
     def _jsondatetime(self, source):
         """ Recursively iterates a JSON dictionary, turning RFC-1123 strings
         into datetime values.
 
+        .. versionchanged:: 0.1.0
+           Datetime conversion was failing on Py2, since 0.0.9 :P
+
         .. versionchanged:: 0.0.9
            support for Python 3.3.
 
         .. versionadded:: 0.0.4
         """
 
+        if sys.version_info[0] == 3:
+            _str_type = str
+        else:
+            _str_type = basestring  # noqa
+
         for k, v in source.items():
             if isinstance(v, dict):
                 self._jsondatetime(v)
-            elif isinstance(v, str):
+            elif isinstance(v, _str_type):
                 try:
                     source[k] = datetime.strptime(v, config.DATE_FORMAT)
                 except:
