@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+from bson import ObjectId
 
 import eve
 import json
@@ -8,8 +9,12 @@ from eve.tests import TestBase
 
 
 class ValidBasicAuth(BasicAuth):
-    def check_auth(self, username, password, allowed_roles, resource):
+    def check_auth(self, username, password, allowed_roles, resource,
+                   method):
         self.user_id = 123
+        # This MUST BE AN OBJECTID, not a string.
+        self._id = ObjectId('deadbeefdeadbeefdeadbeef')
+        self.username = 'admin'
         return username == 'admin' and password == 'secret' and  \
             (allowed_roles == ['admin'] if allowed_roles else True)
 
@@ -19,14 +24,14 @@ class BadBasicAuth(BasicAuth):
 
 
 class ValidTokenAuth(TokenAuth):
-    def check_auth(self, token, allowed_roles, resource):
+    def check_auth(self, token, allowed_roles, resource, method):
         return token == 'test_token' and (allowed_roles == ['admin'] if
                                           allowed_roles else True)
 
 
 class ValidHMACAuth(HMACAuth):
     def check_auth(self, userid, hmac_hash, headers, data, allowed_roles,
-                   resource):
+                   resource, method):
         return userid == 'admin' and hmac_hash == 'secret' and  \
             (allowed_roles == ['admin'] if allowed_roles else True)
 
@@ -232,38 +237,108 @@ class TestUserRestrictedAccess(TestBase):
         # remove the datasource filter to make the whole collection available
         # to a GET request.
         self.resource = self.app.config['DOMAIN'][self.known_resource]
-        del(self.resource['datasource']['filter'])
+        del self.resource['datasource']['filter']
         self.app.set_defaults()
         self.app._add_url_rules()
         self.test_client = self.app.test_client()
         self.valid_auth = [('Authorization', 'Basic YWRtaW46c2VjcmV0')]
         self.invalid_auth = [('Authorization', 'Basic IDontThinkSo')]
         self.field_name = 'auth_field'
-        self.data = {'item1': json.dumps({"ref": "0123456789123456789012345"})}
+        self.data = json.dumps(
+            {'item1': {"ref": "0123456789123456789012345"}}
+        )
         for resource, settings in self.app.config['DOMAIN'].items():
             settings[self.field_name] = 'username'
+        self.resource['public_methods'] = []
 
     def test_get(self):
-        # now need to remove 'GET' from 'public_methods'
-        # because it overrides `auth_username_field`
-        self.resource['public_methods'].remove('GET')
-
         data, status = self.parse_response(
             self.test_client.get(self.known_resource_url,
                                  headers=self.valid_auth))
         self.assert200(status)
-        # no data has been saved by user 'admin' yet, so we get an empty
-        # resultset back.
+        # no data has been saved by user 'admin' yet,
+        # so assert we get an empty result set back.
         self.assertEqual(len(data['_items']), 0)
 
+        # Add a user belonging to `admin`
+        new_user = self.random_contacts(1)[0]
+        new_user['username'] = 'admin'
+        _db = self.connection[self.app.config['MONGO_DBNAME']]
+        _db.contacts.insert(new_user)
+
+        # Verify that we can retrieve it
+        data2, status2 = self.parse_response(
+            self.test_client.get(self.known_resource_url,
+                                 headers=self.valid_auth))
+        self.assert200(status2)
+        self.assertEqual(len(data2['_items']), 1)
+
+    def test_get_by_auth_field_criteria(self):
+        """ If we attempt to retrieve an object by the same field
+        that is in `auth_field`, then the request is /unauthorized/,
+        and should fail and return 401.
+
+        This test verifies that the `auth_field` does not overwrite
+        a `client_filter` or url param.
+        """
+        data, status = self.parse_response(
+            self.test_client.get(self.user_username_url,
+                                 headers=self.valid_auth))
+        self.assert401(status)
+
+    def test_get_by_auth_field_id(self):
+        """ To test handling of ObjectIds
+        """
+        # set auth_field to `_id`
+        self.app.config['DOMAIN']['users'][self.field_name] = \
+            self.app.config['ID_FIELD']
+
+        data, status = self.parse_response(
+            self.test_client.get(self.user_id_url,
+                                 headers=self.valid_auth))
+        self.assert401(status)
+
+    def test_filter_by_auth_field_id(self):
+        """ To test handling of ObjectIds when using a `where` clause
+        We need to make sure we *match* an object ID when it is the
+        same
+        """
+        # set auth_field to `_id`
+        self.app.config['DOMAIN']['users'][self.field_name] = \
+            self.app.config['ID_FIELD']
+
+        # Retrieving a /different user/ by id returns 401
+        user_url = '/users/'
+        filter_by_id = 'where=_id==ObjectId("%s")'
+        filter_query = filter_by_id % self.user_id
+
+        data, status = self.parse_response(
+            self.test_client.get('%s?%s' % (user_url, filter_query),
+                                 headers=self.valid_auth))
+        self.assert401(status)
+
+        # Create a user account belonging to admin
+        new_user = self.random_contacts(1)[0]
+        new_user['_id'] = ObjectId(self.app.auth._id)
+        new_user['username'] = 'admin'
+        _db = self.connection[self.app.config['MONGO_DBNAME']]
+        _db.contacts.insert(new_user)
+
+        # Retrieving /the same/ user by id returns OK
+        filter_query_2 = filter_by_id % new_user['_id']
+        data2, status2 = self.parse_response(
+            self.test_client.get('%s?%s' % (user_url, filter_query_2),
+                                 headers=self.valid_auth))
+        self.assert200(status2)
+        self.assertEqual(len(data2['_items']), 1)
+
     def test_collection_get_public(self):
         """ Test that if GET is in `public_methods` the `auth_username_field`
         criteria is overruled
         """
         self.resource['public_methods'].append('GET')
         data, status = self.parse_response(
-            self.test_client.get(self.known_resource_url,
-                                 headers=self.valid_auth))
+            self.test_client.get(self.known_resource_url))      # no auth
         self.assert200(status)
         # no data has been saved by user 'admin' yet,
         # but we should get all the other results back
@@ -278,8 +353,6 @@ class TestUserRestrictedAccess(TestBase):
             self.test_client.get(self.item_id_url,
                                  headers=self.valid_auth))
         self.assert200(status)
-        # no data has been saved by user 'admin' yet,
-        # but we should get the other result back
         self.assertEqual(data['_id'], self.item_id)
 
     def test_post(self):
@@ -293,26 +366,28 @@ class TestUserRestrictedAccess(TestBase):
         self.assertEqual(len(data['_items']), 1)
 
     def test_patch(self):
-        changes = {"ref": "9999999999999999999999999"}
+        new_ref = "9999999999999999999999999"
+        changes = {'item1': {"ref": new_ref}}
         data, status = self.post()
-        url = '%s%s/' % (self.known_resource_url, data['item1']['_id'])
+        url = '%s/%s' % (self.known_resource_url, data['item1']['_id'])
         response = self.test_client.get(url, headers=self.valid_auth)
         etag = response.headers['ETag']
         headers = [('If-Match', etag),
-                   ('Content-Type', 'application/x-www-form-urlencoded'),
-                   ('Authorization', 'Basic YWRtaW46c2VjcmV0')]
+                   self.valid_auth[0]]
         response, status = self.parse_response(
-            self.test_client.patch(url, data={'item1': json.dumps(changes)},
-                                   headers=headers))
+            self.test_client.patch(url, data=json.dumps(changes),
+                                   headers=headers,
+                                   content_type='application/json'))
         self.assert200(status)
 
         data, status = self.parse_response(
             self.test_client.get(url, headers=self.valid_auth))
         self.assert200(status)
+        self.assertEqual(data['ref'], new_ref)
 
     def test_delete(self):
         data, status = self.post()
-        url = '%s%s/' % (self.known_resource_url, data['item1']['_id'])
+        url = '%s/%s' % (self.known_resource_url, data['item1']['_id'])
         response = self.test_client.get(url, headers=self.valid_auth)
         etag = response.headers['ETag']
         headers = [('If-Match', etag),
@@ -322,9 +397,8 @@ class TestUserRestrictedAccess(TestBase):
         self.assert200(status)
 
     def post(self):
-        headers = [('Content-Type', 'application/x-www-form-urlencoded'),
-                   ('Authorization', 'Basic YWRtaW46c2VjcmV0')]
         r = self.test_client.post(self.known_resource_url,
                                   data=self.data,
-                                  headers=headers)
+                                  headers=self.valid_auth,
+                                  content_type='application/json')
         return self.parse_response(r)
