@@ -4,7 +4,8 @@ from unittest import TestCase
 from bson import ObjectId
 from datetime import datetime
 from eve.io.mongo.parser import parse, ParseError
-from eve.io.mongo import Validator
+from eve.io.mongo import Validator, Mongo
+from eve.utils import config
 from cerberus.errors import ERROR_BAD_TYPE
 
 
@@ -107,3 +108,45 @@ class TestMongoValidator(TestCase):
         schema = {'a_field': {'type': 'string'}}
         v = Validator(schema)
         self.assertTrue(v.transparent_schema_rules, True)
+
+
+class TestMongoDriver(TestCase):
+    def test_combine_queries(self):
+        mongo = Mongo(None)
+        query_a = {'username': {'$exists': True}}
+        query_b = {'username': 'mike'}
+        combined = mongo.combine_queries(query_a, query_b)
+        self.assertEqual(
+            combined,
+            {'$and': [{'username': {'$exists': True}}, {'username': 'mike'}]}
+        )
+
+    def test_get_value_from_query(self):
+        mongo = Mongo(None)
+        simple_query = {config.ID_FIELD: 'abcdef012345678901234567'}
+        compound_query = {'$and': [
+            {'username': {'$exists': False}},
+            {config.ID_FIELD: 'abcdef012345678901234567'}
+        ]}
+        self.assertEqual(mongo.get_value_from_query(simple_query,
+                                                    config.ID_FIELD),
+                         'abcdef012345678901234567')
+        self.assertEqual(mongo.get_value_from_query(compound_query,
+                                                    config.ID_FIELD),
+                         'abcdef012345678901234567')
+
+    def test_query_contains_field(self):
+        mongo = Mongo(None)
+        simple_query = {config.ID_FIELD: 'abcdef012345678901234567'}
+        compound_query = {'$and': [
+            {'username': {'$exists': False}},
+            {config.ID_FIELD: 'abcdef012345678901234567'}
+        ]}
+        self.assertTrue(mongo.query_contains_field(simple_query,
+                                                   config.ID_FIELD))
+        self.assertFalse(mongo.query_contains_field(simple_query,
+                                                    'fake-field'))
+        self.assertTrue(mongo.query_contains_field(compound_query,
+                                                   config.ID_FIELD))
+        self.assertFalse(mongo.query_contains_field(compound_query,
+                                                    'fake-field'))
