@@ -1,5 +1,6 @@
 #import unittest
 from eve.tests import TestBase
+from eve.tests.test_settings import MONGO_DBNAME
 from eve import STATUS_OK, LAST_UPDATED, ID_FIELD
 import simplejson as json
 
@@ -172,6 +173,7 @@ class TestPatch(TestBase):
             self.assertEqual(db_values[i], test_values[i])
 
     def test_patch_with_post_override(self):
+        # a POST request with PATCH override turns into a PATCH request
         r = self.perform_patch_with_post_override('prog', 1)
         self.assert200(r.status_code)
 
@@ -184,7 +186,7 @@ class TestPatch(TestBase):
         return r
 
     def perform_patch_with_post_override(self, field, value):
-        headers = [('X-HTTP-Method-Override', 'True'),
+        headers = [('X-HTTP-Method-Override', 'PATCH'),
                    ('If-Match', self.item_etag),
                    ('Content-Type', 'application/x-www-form-urlencoded')]
         return self.test_client.post(self.item_id_url,
@@ -266,6 +268,33 @@ class TestPatch(TestBase):
                                headers=[('If-Match', self.item_etag)])
         self.assert500(status)
 
+    def test_patch_missing_standard_date_fields(self):
+        """Documents created outside the API context could be lacking the
+        LAST_UPDATED and/or DATE_CREATED fields.
+        """
+        # directly insert a document, without DATE_CREATED e LAST_UPDATED
+        # values.
+        contacts = self.random_contacts(1, False)
+        ref = 'test_update_field'
+        contacts[0]['ref'] = ref
+        _db = self.connection[MONGO_DBNAME]
+        _db.contacts.insert(contacts)
+
+        # now retrieve same document via API and get its etag, which is
+        # supposed to be computed on default DATE_CREATED and LAST_UPDATAED
+        # values.
+        response, status = self.get(self.known_resource, item=ref)
+        etag = response['etag']
+        _id = response['_id']
+
+        # attempt a PATCH with the new etag.
+        field = "ref"
+        test_value = "X234567890123456789012345"
+        changes = {'key1': json.dumps({field: test_value})}
+        r, status = self.patch('%s/%s' % (self.known_resource_url, _id),
+                               data=changes, headers=[('If-Match', etag)])
+        self.assert200(status)
+
     def assertPatchResponse(self, response, key, item_id):
         self.assertTrue(key in response)
         k = response[key]
