@@ -10,7 +10,7 @@ ID_FIELD = '_id'
 SERVER_NAME = 'localhost:5000'
 
 RESOURCE_METHODS = ['GET', 'POST', 'DELETE']
-ITEM_METHODS = ['GET', 'PATCH', 'DELETE']
+ITEM_METHODS = ['GET', 'PATCH', 'DELETE', 'PUT']
 ITEM_CACHE_CONTROL = ''
 ITEM_LOOKUP = True
 ITEM_LOOKUP_FIELD = ID_FIELD
@@ -84,6 +84,7 @@ users['datasource'] = {'source': 'contacts',
 users['schema']['username'] = {'type': 'string', 'required': True}
 users['resource_methods'] = ['DELETE', 'POST', 'GET']
 users['item_title'] = 'user'
+users['additional_lookup']['field'] = 'username'
 
 invoices = {
     #'item_lookup': False,
