@@ -12,12 +12,15 @@ redis = Redis()
 
 
 class TAuth(TokenAuth):
-    def check_auth(self, token, allowed_roles, resource):
+    def check_auth(self, token, allowed_roles, resource, method):
         return token == 'token'
 
 class Auth(BasicAuth):
-    def check_auth(self, username, password, allowed_roles, resource):
-        print username
+    def check_auth(self, username, password, allowed_roles, resource, method):
+        if resource is None:
+            print "NONE"
+            return True
+        #print username
         self.user_id = username
         return username == 'token'
         #return username == 'admin' and password == 'secret'
@@ -62,7 +65,7 @@ def posting_r(documents):
     print ("posting to resource contacts")
 
 
-app = Eve(validator=Validator, redis=redis, auth=Auth)
+app = Eve(validator=Validator, redis=redis)
 #app.config['DOMAIN']['contacts']['auth_username_field'] = 'name'
 #app.on_get_contacts += resource_gotit
 #app.on_get += gotit
@@ -74,5 +77,5 @@ app = Eve(validator=Validator, redis=redis, auth=Auth)
 
 if __name__ == '__main__':
     #app.events.on_getting += pre
-    #app.events.on_get += post
+    app.on_GET += gotit
     app.run(debug=True)
